#! /usr/bin/env python3
# coding: utf8

# Convertisseur Alias-Wavefront -> SGML 3D à usage
# particulier: il travaille sur un objet 3D unique, mais texturé.
# Il ne tient donc pas compte de la structuration éventuelle du
# fichier .obj en objets et sous-objets, en revanche il corrige
# l'éventuelle différence de numérotation entre les points 3D et
# les points UV correspondants, y compris quand ça l'oblige à dupliquer
# certains points 3D.

import sys
from os import system

# --- Definitions de classes

class Point3D:
    def __init__(self, xraysArg = 0.0, yankeeArg = 0.0, zuluArg = 0.0):
        self.xrays = xraysArg
        self.yankee = yankeeArg
        self.zulu = zuluArg
        self.nuvoNum = -1

class PointUV:
    def __init__(self, uniformArg, victorArg):
        self.uniform = uniformArg
        self.victor = victorArg

class VertexFaceTexturee:
    def __init__(self, numPoint3DArg, numPointUVArg):
        self.numPoint3D = numPoint3DArg
        self.numPointUV = numPointUVArg

class Fichier:
    def __init__(self, nomFichierArg, boolEcriture):
        self.nomFichier = nomFichierArg
        if boolEcriture:
            self.fichier = open(self.nomFichier, 'wb') # 'b' pour des octets
            self.fichier.seek(0, 0) # Se place au debut du fichier
            self.longueur = 0
        else:
            self.fichier = open(self.nomFichier, 'rb') # 'b' pour des octets
            self.fichier.seek(0, 2) # Se place a la fin du fichier
            self.longueur = self.fichier.tell()
        self.index = 0

    def close (self):
        self.fichier.close()

    def deByteStringAChaineUnicode (self, monByteString):
        chaineUnicode = monByteString.decode(encoding='UTF-8')
        return chaineUnicode

    def deChaineUnicodeAByteString (self, chaineUnicode):
        monByteString = chaineUnicode.encode(encoding='UTF-8')
        return monByteString

    def ecritUneLigne (self, ligneAEcrire):
        ligneAEcrire2 = self.deChaineUnicodeAByteString(ligneAEcrire)
        for numOctet in range(len(ligneAEcrire2)):
            octet = ligneAEcrire2[numOctet:numOctet + 1]
            # La ligne precedente a l'air amphigourique, mais
            # ligneAEcrire2[numOctet:numOctet + 1] est de type "bytes",
            # alors que ligneAEcrire2[numOctet] serait de type "int"
            self.ecritUnOctet(octet)
        self.ecritUnOctet(b'\n')

    def ecritUnOctet (self, signe):
        self.fichier.seek(self.index, 0)
        self.fichier.write(signe)
        self.index += 1
        self.longueur += 1

    def interromptLecture (self):
        self.close()

    def litUneLigne (self):
        octetLu = ''
        ligneLue = b"" # Soit un bytestring vide
        finDeLigne = 0 # false

        while self.index < self.longueur and not finDeLigne:
            octetLu = self.litUnOctet(self.index)
            if octetLu == b'\n':
                finDeLigne = 1 # true
            else:
                ligneLue += octetLu

        ligneLue2 = self.deByteStringAChaineUnicode(ligneLue)
        return ligneLue2

    def litUnOctet (self, numOctet):
        self.fichier.seek(numOctet, 0)
        octet = self.fichier.read(1)
        self.index += 1
        return octet

    def reprendLecture (self):
        self.fichier = open(self.nomFichier, 'r')
        self.fichier.seek(self.index, 0) # Se place ou on s'etait arrete

    def seek (self, numOctet):
        self.fichier.seek(numOctet, 0)
        self.index = numOctet

# --- Definitions de fonctions

def principal (argv):
    nbArgs = len(sys.argv)

    if nbArgs >= 4:
        nomFichALire = sys.argv[1]
        nomFichAEcrire = sys.argv[2]
        nomTexture = sys.argv[3]
    else:
        print("*** Il faut donner en arg. le nom des fichiers lu et ecrit,")
        print("    et le nom de la texture.")
        sys.exit(1)

    # D'abord, lecture brute des informations du fichier
    fichALire = Fichier(nomFichALire, 0)

    mesPoints3D = []
    mesPointsUV = []
    mesFaces = []
    
    while fichALire.index < fichALire.longueur:
        ligneLue = fichALire.litUneLigne()
        if len(ligneLue) >= 2 and ligneLue[0:2] == "v ":
            mesBazars = ligneLue.split(' ')
            xrays = float(mesBazars[1])
            yankee = float(mesBazars[3]) * -1.0
            zulu = float(mesBazars[2])
            monPoint3D = Point3D(xrays, yankee, zulu)
            mesPoints3D.append(monPoint3D)
        elif len(ligneLue) >= 3 and ligneLue[0:3] == "vt ":
            mesBazars = ligneLue.split(' ')
            uniform = float(mesBazars[1])
            victor = float(mesBazars[2])
            monPointUV = PointUV(uniform, victor)
            mesPointsUV.append(monPointUV)
        elif len(ligneLue) >= 2 and ligneLue[0:2] == "f ":
            mesBazars = ligneLue.split(' ')
            verticesFace = []
            for numBazar in range(len(mesBazars) - 1):
                mesBazarsBis = (mesBazars[numBazar + 1]).split('/')
                numPoint3D = int(mesBazarsBis[0])
                numPointUV = int(mesBazarsBis[1])
                monVertex = VertexFaceTexturee(numPoint3D, numPointUV)
                verticesFace.append(monVertex)
            mesFaces.append(verticesFace)
    fichALire.close()

    # Unification des points 3D et UV par recopiage des coord. 3D dans un nouveau tableau
    nbPointsUV = len(mesPointsUV)
    mesPoints3D_bis = []

    # Dimensionnement du nouveau tableau. Oui, carrément, comme en C!
    for numPointUV in range(nbPointsUV):
        monPoint3D = Point3D()
        mesPoints3D_bis.append(monPoint3D)
    
    for numFace in range(len(mesFaces)):
        monTableau = mesFaces[numFace]

        for numVertex in range(len(monTableau)):
            monVertex = monTableau[numVertex]
            numPoint3D_debut1 = monVertex.numPoint3D # Wavefront numérote à partir de 1
            numPointUV_debut1 = monVertex.numPointUV # Wavefront numérote à partir de 1
            vraiNumPoint3D = numPoint3D_debut1 - 1
            vraiNumPointUV = numPointUV_debut1 - 1
            mesPoints3D_bis[vraiNumPointUV].xrays = mesPoints3D[vraiNumPoint3D].xrays
            mesPoints3D_bis[vraiNumPointUV].yankee = mesPoints3D[vraiNumPoint3D].yankee
            mesPoints3D_bis[vraiNumPointUV].zulu = mesPoints3D[vraiNumPoint3D].zulu

    # Stockage
    fichAEcrire = Fichier(nomFichAEcrire, 1)

    ligneAEcrire = "<texture>"
    fichAEcrire.ecritUneLigne(ligneAEcrire)
    ligneAEcrire = nomTexture
    fichAEcrire.ecritUneLigne(ligneAEcrire)
    ligneAEcrire = "</texture>"
    fichAEcrire.ecritUneLigne(ligneAEcrire)

    ligneAEcrire = "<points3D>"
    fichAEcrire.ecritUneLigne(ligneAEcrire)
    for numPoint3D in range(len(mesPoints3D_bis)):
        monPoint = mesPoints3D_bis[numPoint3D]
        ligneAEcrire = "%d,%f,%f,%f" % (numPoint3D + 1, \
                                        monPoint.xrays, monPoint.yankee, monPoint.zulu)
        fichAEcrire.ecritUneLigne(ligneAEcrire)
    ligneAEcrire = "</points3D>"
    fichAEcrire.ecritUneLigne(ligneAEcrire)
    
    ligneAEcrire = "<pointsUV>"
    fichAEcrire.ecritUneLigne(ligneAEcrire)
    for numPointUV in range(len(mesPointsUV)):
        monPoint = mesPointsUV[numPointUV]
        ligneAEcrire = "%d,%f,%f" % (numPointUV + 1, monPoint.uniform, monPoint.victor)
        fichAEcrire.ecritUneLigne(ligneAEcrire)
    ligneAEcrire = "</pointsUV>"
    fichAEcrire.ecritUneLigne(ligneAEcrire)

    ligneAEcrire = "<faces>"
    fichAEcrire.ecritUneLigne(ligneAEcrire)
    for numFace in range(len(mesFaces)):
        ligneAEcrire = ""
        monTableau = mesFaces[numFace]
        for numVertex in range(len(monTableau)):
            monVertex = monTableau[numVertex]
            # ligneAEcrire += "%d (%d)" % (monVertex.numPoint3D, monVertex.numPointUV)
            ligneAEcrire += "%d" % (monVertex.numPointUV)
            if numVertex < len(monTableau) - 1:
                ligneAEcrire += ','
        fichAEcrire.ecritUneLigne(ligneAEcrire)
    ligneAEcrire = "</faces>"
    fichAEcrire.ecritUneLigne(ligneAEcrire)
    
    fichAEcrire.close()

# --- Fin des definitions de classes et de fonctions,
# --- debut du programme principal

principal(sys.argv)
